<?php

namespace jrosset\CommandLine\Argument\Option;

use InvalidArgumentException;

/**
 * A "flag" option argument
 *
 * The output value is a boolean
 */
class Flag extends AbstractOptionArgument {
    /**
     * Create a new flag option argument
     *
     * By default, the {@see AbstractArgument::$varName variable name} is the same as the {@see AbstractArgument::$name argument in help}
     *
     * @param string      $name        The argument name in help
     * @param string|null $description The argument name in documentation
     * @param bool        $default     The argument default value
     * @param string|null $tagLong     The long tag. If null, deduce from argument name <b>$name</b> (see {@see AbstractOptionArgument::convertNameToTagLong()}
     * @param string|null $tagShort    The short tag
     */
    public function __construct (string $name, ?string $description, bool $default, ?string $tagLong = null, ?string $tagShort = null) {
        parent::__construct($name, $description, $tagLong, $tagShort);
        $this->setDefault($default);
    }

    /**
     * Set default value for the argument
     *
     * @param boolean $default The default value for the argument (boolean)
     *
     * @return $this
     *
     * @throws InvalidArgumentException If the default value provided is not valid (boolean)
     */
    public function setDefault ($default = false): self {
        if (!is_bool($default)) {
            throw new InvalidArgumentException('The default value must be a boolean');
        }

        parent::setDefault($default);
        return $this;
    }
}