<?php
/**
 * Déclaration de la classe CommandLine\Argument\Parser\EnumParser
 */
namespace jrosset\CommandLine\Argument\Parser;

use InvalidArgumentException;
use RangeException;

/**
 * Parseur vers une liste de valeurs possible (chaine de caractère).
 *
 * @package      CommandLine\Argument\Parser
 *
 * @noinspection PhpUnused
 */
class EnumParser implements IParser {
    /**
     * @var string[] Liste des valeurs autorisées
     */
    protected array $values;

    /**
     * Crée un nouveau parseur
     *
     * @param string[] $values La liste des valeurs autorisées
     */
    public function __construct (array $values) {
        $this->setValues($values);
    }

    /**
     * @inheritDoc
     */
    public function parseValue (string $val) {
        if (!in_array($val, $this->values)) {
            throw new RangeException('La valeur ne fait partie de liste des valeurs autorisées (' . implode(', ', $this->getValues()) . ') : ' . $val);
        }

        return $val;
    }

    /**
     * @inheritDoc
     */
    public function getValueDescription (): string {
        return 'enum(' . implode(',', $this->getValues()) . ')';
    }

    /**
     * La liste des valeurs autoriséees.
     *
     * @return string[] La liste des valeurs
     *
     * @see $values
     */
    public function getValues (): array {
        return $this->values;
    }
    /**
     * Définit la liste des valeurs autorisées.
     *
     * @param string[] $values La liste des valeurs
     *
     * @return $this
     * @see $values
     */
    public function setValues (array $values): self {
        if (!is_array($values)) {
            throw new InvalidArgumentException('La liste de valeurs n\'est pas un tableau');
        }

        if (count($values) == 0) {
            throw new InvalidArgumentException('La liste de valeurs doit avoir au moins un élément');
        }

        $this->values = $values;
        return $this;
    }
}