<?php

namespace jrosset\CommandLine\Argument\Value;

use jrosset\CommandLine\Argument\IArgumentValueDescription;
use jrosset\CommandLine\Argument\Parser\IParser;

/**
 * Argument de type "valeur"
 *
 * @package CommandLine\Argument\Value
 */
class Value extends AbstractValueArgument implements IArgumentValueDescription {
    /**
     * @var IParser The values parser
     */
    private IParser $valuesParser;

    /**
     * Create a new value argument
     *
     * By default, the {@see AbstractArgument::$varName variable name} is the same as the {@see AbstractArgument::$name argument in help}
     *
     * @param string      $name         The argument name in help
     * @param string|null $description  The argument name in documentation
     * @param IParser     $valuesParser The value parser
     * @param boolean     $optional    Optional argument ?
     * @param mixed       $default     The default value
     */
    public function __construct (string $name, ?string $description, IParser $valuesParser, bool $optional = false, $default = null) {
        parent::__construct($name, $description, $optional, $default);
        $this->setValuesParser($valuesParser);
    }

    /**
     * @inheritDoc
     */
    public function getValueDescription (): string {
        return $this->valuesParser->getValueDescription();
    }

    /**
     * The values parser
     *
     * @return IParser The values parser
     */
    public function getValuesParser (): IParser {
        return $this->valuesParser;
    }
    /**
     * Set the values parser
     *
     * @param IParser $valuesParser The values parser
     *
     * @return $this
     */
    public function setValuesParser (IParser $valuesParser): self {
        $this->valuesParser = $valuesParser;
        return $this;
    }
}