<?php

namespace jrosset\CommandLine;

use InvalidArgumentException;

/**
 * An object-oriented class for an array
 *
 * @template T
 */
class ArrayClass {
    /**
     * @var T[] The internal array
     */
    private array $array;

    /**
     * Create an empty array
     */
    public function __construct () {
        $this->setAll([]);
    }

    /**
     * Get all elements of the array
     *
     * @return T[] all elements of the array
     */
    public function getAll (): array {
        return $this->array;
    }
    /**
     * <b>Replace</b> all elements of the array
     *
     * @param T[] $array The new list of elements
     *
     * @return $this
     *
     * @throws InvalidArgumentException If an element of the array is not valid
     *
     * @see ArrayClass::add()
     */
    public function setAll (array $array): self {
        self::checkArrayType($array);

        $this->array = [];
        foreach ($array as $element) {
            $this->add($element);
        }
        return $this;
    }

    /**
     * Add one or an array of elements
     *
     * @param T|T[] $elements The element(s) to add
     *
     * @return $this
     *
     * @throws InvalidArgumentException If an element of the array is not valid
     */
    public function add ($elements): self {
        if (!is_array($elements)) {
            $elements = [$elements];
        }
        $this->checkArrayType($elements);

        $realArray = [];
        foreach ($elements as $element) {
            $key = $this->getKeyForElement($element);
            if ($key === null) {
                $realArray[] = $element;
            }
            else {
                $realArray[$key] = $element;
            }
        }

        $this->array = array_merge($this->array, $realArray);
        return $this;
    }

    /**
     * Check if a element exists from his key
     *
     * @param int|string $elementKey The element key to check
     *
     * @return bool True if the element exists, else False
     */
    public function exists ($elementKey): bool {
        return isset($this->getAll()[$elementKey]);
    }

    /**
     * Check if all elements of the parameter array are valid
     *
     * @param T[] $array The array to check
     *
     * @throws InvalidArgumentException If an element of the array is not valid
     *
     * @see ArrayClass::$checker
     */
    protected function checkArrayType (array $array): void {
        foreach ($array as $arrayKey => $arrayElement) {
            $this->checkArrayElementType($arrayKey, $arrayElement);
        }
    }
    /**
     * Check if the element of the array is valid
     *
     * @param int|string $key The key of the element
     * @param T $element The element to check
     *
     * @throws InvalidArgumentException If the element is not valid
     */
    protected function checkArrayElementType ($key, $element): void {}

    /**
     * Get the key of an element. Null if auto-generated
     *
     * @param T $element The element
     *
     * @return int|string|null The key or Null
     * @noinspection PhpReturnDocTypeMismatchInspection
     */
    protected function getKeyForElement ($element) {
        return null;
    }
}