<?php

namespace jrosset\CommandLine\ReturnCode;

/**
 * A return code
 */
class ReturnCode implements IReturnCode {
    /**
     * @var int The returned code
     */
    private int $code;
    /**
     * @var string|null The description of the return code
     */
    private ?string $description;

    /**
     * Create a new return code
     *
     * @param int         $code        The returned code
     * @param string|null $description The description of the return code
     */
    public function __construct (int $code, ?string $description = null) {
        $this->setCode($code);
        $this->setDescription($description);
    }

    /**
     * @inheritDoc
     */
    public function getCode (): int {
        return $this->code;
    }
    /**
     * Set the returned code
     *
     * @param int $code The returned code
     *
     * @return $this
     */
    public function setCode (int $code): self {
        $this->code = $code;
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getDescription (): ?string {
        return $this->description;
    }
    /**
     * Set the description of the return code
     *
     * @param string|null $description The description of the return code
     *
     * @return $this
     */
    public function setDescription (?string $description): self {
        $this->description = $description;
        return $this;
    }
}